@@ -1,5 +1,29 @@
 import json
 import ee
+import types
+import atexit
+
+exiting = False
+
+def set_exiting():
+	global exiting
+	exiting = True
+
+atexit.register(set_exiting)
+
+def encodeInput(inputVal):
+	if(isinstance(inputVal, types.FunctionType)):
+		return {'type':'function','value':id(inputVal)}
+	if(isinstance(inputVal, ee.computedobject.ComputedObject)):
+		return {'type':'ee','ee_type':inputVal.name(),'value':json.dumps(ee.serializer.encode(inputVal))};
+	return {'type':'other','value':json.dumps(ee.serializer.encode(inputVal))};
+
+def decodeInput(inputVal):
+	if(inputVal['type']=='function'):
+		raise NotImplementedError("This function has not been implemented yet due to the missing counterpart in JavaScript.");
+	if(inputVal['type']=='ee'):
+		return getattr(ee,inputVal['ee_type'])(ee.deserializer.fromJSON(inputVal["value"]));
+	return ee.deserializer.fromJSON(inputVal["value"]);
 
 class externalEEjs():
 	libInterface=None;
@@ -27,13 +51,13 @@ class externalEEjs():
 				def localFunc(*argl,**args):
 					argl=list(argl);
 					for key in args.keys():
-						args[key]=json.dumps(ee.serializer.encode(args[key]));
+						args[key]=encodeInput(args[key]);
 					for x in range(len(argl)):
-						argl[x]=json.dumps(ee.serializer.encode(argl[x]));
+						argl[x]=encodeInput(argl[x]);
 					self.nodeSocket.send_string(json.dumps({'type':'call','lib':self.nodeID,'functionName':functionName,'args':args,'argl':argl}))
 					answer=json.loads(self.nodeSocket.recv())
 					if 'payload' in answer:
-						return ee.deserializer.fromJSON(answer['payload']);
+						return decodeInput(answer['payload']);
 					else:
 						return
 				return localFunc;
@@ -41,7 +65,7 @@ class externalEEjs():
 				self.nodeSocket.send_string(json.dumps({'type':'call','lib':self.nodeID,'functionName':functionName,'args':None}))
 				answer=json.loads(self.nodeSocket.recv())
 				if 'payload' in answer:
-					return ee.deserializer.fromJSON(answer['payload']);
+					return decodeInput(answer['payload']);
 				else:
 					return;
 
@@ -51,11 +75,14 @@ class externalEEjs():
 			return self.availability.keys();
 
 		def __del__(self):
+			pass
 			self.libInterface=None;
 			self.initialized=False;
 			self.nodeSocket.send_string(json.dumps({'type':'unload','lib':self.nodeID}))
 			self.nodeID=None;
-			answer=json.loads(self.nodeSocket.recv())
+			global exiting
+			if not exiting:
+				answer=json.loads(self.nodeSocket.recv())
 
 	def __init__(self,soket,libPath):
 		soket.send_string(json.dumps({'type':'load','lib':libPath}))
@@ -64,7 +91,3 @@ class externalEEjs():
 			raise oeelInitFailed();
 		self.libInterface=self.callArgument(answer,soket);
 		self.initialized=True;
-
-
-		
-		
\ No newline at end of file
